﻿//-----------------------------------------------------------------------------
// <copyright file="DeviceDescription.cs" company="eQ-3 Entwicklung GmbH">
//   Copyright (c) 2010
// </copyright>
//-----------------------------------------------------------------------------

namespace ListDevices
{
  using System;
  using System.Collections.Generic;
  using System.Text;

  using CookComputing.XmlRpc;

  /// <summary>
  /// Enthält Informationen über ein logisches Gerät
  /// </summary>
  public class DeviceDescription
  {
    /// <summary>
    /// Liefert oder setzt den Typ des logischen Geräts.
    /// Entspricht bei Geräten der Kurzbezeichnung. 
    /// </summary>
    [XmlRpcMember("TYPE")]
    public string Type
    {
      get;
      set;
    }

    /// <summary>
    /// Liefert oder setzt die Seriennummer des logischen Geräts.
    /// </summary>
    [XmlRpcMember("ADDRESS")]
    public string Address
    {
      get;
      set;
    }

    /// <summary>
    /// Liefert oder setzt die Seriennummer des übergeordneten Geräts.
    /// Ist bei physikalischen Geräten "".
    /// </summary>
    [XmlRpcMember("PARENT")]
    public string Parent
    {
      get;
      set;
    }

    /// <summary>
    /// Liefert oder setzt die Seriennummern der untergeorneten Kanäle.
    /// Ist nur bei physikalischen Geräten vorhanden.
    /// </summary>
    [XmlRpcMissingMapping(MappingAction.Ignore), XmlRpcMember("CHILDREN")]
    public string[] Children
    {
      get;
      set;
    }

    /// <summary>
    /// Liefert oder setzt den Typ des übergeordneten Geräts.
    /// Ist nur bei Kanälen vorhanden.
    /// </summary>
    [XmlRpcMissingMapping(MappingAction.Ignore), XmlRpcMember("PARENT_TYPE")]
    public string ParentType
    {
      get;
      set;
    }

    /// <summary>
    /// Liefert oder setzt die Firmware-Version.
    /// Ist nur bei physikalischen Geräten vorhanden.
    /// </summary>
    [XmlRpcMissingMapping(MappingAction.Ignore), XmlRpcMember("FIRMWARE")]
    public string Firmware
    {
      get;
      set;
    }

    /// <summary>
    /// Kategorie (Sender / Empfänger / nicht verknüpfbar).
    /// Ist nur bei Kanälen vorhanden.
    /// </summary>
    [XmlRpcMissingMapping(MappingAction.Ignore), XmlRpcMember("DIRECTION")]
    public int Direction
    {
      get;
      set;
    }

    /// <summary>
    /// Ermittelt, ob das logische Gerät ein
    /// physikalisches Gerät ist
    /// </summary>
    /// <returns>
    /// True, falls das logische Gerät
    /// ein physikalisches ist
    /// </returns>
    public bool IsDevice()
    {
      return string.IsNullOrEmpty(this.Parent);
    }

    /// <summary>
    /// Ermittelt, ob das logische Gerät ein
    /// Kanal ist
    /// </summary>
    /// <returns>
    /// True, falls das logische Gerät ein
    /// Kanal ist
    /// </returns>
    public bool IsChannel()
    {
      return !this.IsDevice();
    }

    /// <summary>
    /// Liefert die Zeichenkettendarstellung
    /// des logische Geräts
    /// </summary>
    /// <returns>
    /// Zeichenkettendarstellung
    /// des logische Geräts
    /// </returns>
    public override string ToString()
    {
      return this.Address;
    }

    /// <summary>
    /// Liefert die Bezeichnung der
    /// Kategorie des Logischen Geräts
    /// </summary>
    /// <returns>
    /// Bezeichnung der
    /// Kategorie des Logischen Geräts
    /// </returns>
    public string GetCategory()
    {
      switch (this.Direction)
      {
        case 0:
          return "nicht verknüpfbar";
        case 1:
          return "Sender (Sensor)";
        case 2:
          return "Empfänger (Aktor)";
        default:
          return string.Empty;
      }
    }
  }
}
