﻿//-----------------------------------------------------------------------------
// <copyright file="MainForm.cs" company="eQ-3 Entwicklung GmbH">
//   Copyright (c) 2010
// </copyright>
//-----------------------------------------------------------------------------

namespace ListDevices
{
  using System;
  using System.Windows.Forms;

  using CookComputing.XmlRpc;

  /// <summary>
  /// Hauptfenster der Anwendung
  /// </summary>
  public partial class MainForm : Form
  {
    /// <summary>
    /// Timeout, nachdem der Proxy den
    /// Verbindungsversuch abbricht
    /// </summary>
    private const int Timeout = 30000;

    /// <summary>
    /// Liste der logischen HomeMatic Geräte
    /// </summary>
    private DeviceDescription[] devices;

    /// <summary>
    /// Erstellt und initialisiert das
    /// Hauptfenster der Anwendung
    /// </summary>
    public MainForm()
    {
      this.InitializeComponent();
      this.devices = new DeviceDescription[0];
    }

    /// <summary>
    /// Reagiert auf den "Verbinden"-Button.
    /// Ermittelt die verfügbaren HomeMatic Geräte.
    /// </summary>
    /// <param name="sender">Sender des Ereignisses</param>
    /// <param name="e">Parameter des Ereignisses</param>
    private void ConnectButton_Click(object sender, EventArgs e)
    {
      try
      {
        IHomeMaticProxy proxy = XmlRpcProxyGen.Create<IHomeMaticProxy>();
        proxy.Timeout = MainForm.Timeout;
        proxy.Url = this.urlTextBox.Text;
        this.devices = proxy.ListDevices();
      }
      catch
      {
        MessageBox.Show("Die Verbindung konnte nicht aufgebaut werden.", "Fehler");
        this.devices = new DeviceDescription[0];
      }

      this.UpdateDevicesTreeView();
    }

    /// <summary>
    /// Aktualisiert die baumartige Ansicht der
    /// HomeMatic Geräte
    /// </summary>
    private void UpdateDevicesTreeView()
    {
      this.devicesTreeView.Nodes.Clear();
      TreeNode rootNode = new TreeNode(this.urlTextBox.Text);
      foreach (DeviceDescription device in this.devices)
      {
        if (device.IsDevice())
        {
          TreeNode node = new TreeNode(device.Address);
          node.Tag = device;
          rootNode.Nodes.Add(node);
        }
      }

      this.devicesTreeView.Nodes.Add(rootNode);
      this.devicesTreeView.ExpandAll();
      this.devicesTreeView.SelectedNode = rootNode;
    }

    /// <summary>
    /// Reagiert auf die Auswahl eines HomeMatic Geräts.
    /// Aktualisiert die Detailansicht.
    /// </summary>
    /// <param name="sender">Sender des Ereignisses</param>
    /// <param name="e">Parameter des Ereignisses</param>
    private void DevicesTreeView_AfterSelect(object sender, TreeViewEventArgs e)
    {
      DeviceDescription device = this.GetSelectedDevice();

      if (device != null)
      {
        this.addressTextBox.Text = device.Address;
        this.typeTextBox.Text = device.Type;
        this.firmwareTextBox.Text = device.Firmware;

        this.channelsListView.Items.Clear();
        foreach (DeviceDescription channel in this.devices)
        {
          if (channel.Parent == device.Address)
          {
            ListViewItem item = new ListViewItem(channel.Address);
            item.SubItems.Add(channel.Type);
            item.SubItems.Add(channel.GetCategory());
            this.channelsListView.Items.Add(item);
          }
        }

        this.channelsListView.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
      }

      this.backgroundPanel.Visible = device != null;
    }

    /// <summary>
    /// Liefert das aktuell ausgewählte Gerät
    /// </summary>
    /// <returns>
    /// Ausgewähltes Gerät oder null, falls kein Gerät ausgewählt ist
    /// </returns>
    private DeviceDescription GetSelectedDevice()
    {
      if (this.devicesTreeView.SelectedNode != null && this.devicesTreeView.SelectedNode.Tag is DeviceDescription)
      {
        return (DeviceDescription)this.devicesTreeView.SelectedNode.Tag;
      }
      else
      {
        return null;
      }
    }
  }
}
